SystemOrganization addCategory: #MemoryMonitor!

Morph subclass: #MMGraph
	instanceVariableNames: 'probes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MemoryMonitor'!

!MMGraph methodsFor: 'accessing' stamp: 'Alexandre Bergel 5/25/2010 20:12'!
addProbe: probe
	probes add: probe! !

!MMGraph methodsFor: 'updating' stamp: 'Alexandre Bergel 5/25/2010 18:27'!
clear
	probes do: #clear! !

!MMGraph methodsFor: 'as yet unclassified' stamp: 'Alexandre Bergel 5/25/2010 15:08'!
colorForBackground
	^ Color gray! !

!MMGraph methodsFor: 'drawing' stamp: 'Alexandre Bergel 5/25/2010 18:30'!
drawOn: aCanvas
	aCanvas fillRectangle: self bounds color: self colorForBackground.
	probes do: [ :probe | probe renderOnCanvas: aCanvas bounds: self bounds ]! !

!MMGraph methodsFor: 'initialization' stamp: 'Alexandre Bergel 5/31/2010 15:23'!
initialize
	super initialize.
	probes := OrderedCollection new.
! !

!MMGraph methodsFor: 'accessing-readonly' stamp: 'Alexandre Bergel 5/25/2010 20:25'!
numberOfProbes
	^ probes size! !

!MMGraph methodsFor: 'stepping and presenter' stamp: 'Alexandre Bergel 5/25/2010 18:29'!
step
	self changed.
	probes do: [:probe | probe updateValue ]! !

!MMGraph methodsFor: 'testing' stamp: 'Alexandre Bergel 5/25/2010 15:08'!
stepTime
	"Answer the desired time between steps in milliseconds."

	^ 500! !

StringMorph subclass: #MMMemoryBoundsIndicadorMorph
	instanceVariableNames: 'memoryProbe'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MemoryMonitor'!

!MMMemoryBoundsIndicadorMorph methodsFor: 'initialization' stamp: 'Alexandre Bergel 5/31/2010 22:55'!
initialize 
	super initialize.
	memoryProbe := MUsedMemoryProbe new! !

!MMMemoryBoundsIndicadorMorph methodsFor: 'accessing' stamp: 'Alexandre Bergel 5/31/2010 22:55'!
memoryProbe
	^ memoryProbe! !

!MMMemoryBoundsIndicadorMorph methodsFor: 'accessing' stamp: 'Alexandre Bergel 5/31/2010 22:55'!
memoryProbe: anObject
	memoryProbe := anObject! !

!MMMemoryBoundsIndicadorMorph methodsFor: 'stepping and presenter' stamp: 'Alexandre Bergel 5/31/2010 23:00'!
step
	| min max |
	min := (memoryProbe computeMinValue / (1024 * 1024)) asFloat roundTo: 0.1.
	max := (memoryProbe computeMaxValue / (1024 * 1024)) asFloat roundTo: 0.1.
	self contents: 'Memory min: ', min printString , 'M max: ', max printString, 'M'! !

!MMMemoryBoundsIndicadorMorph methodsFor: 'testing' stamp: 'Alexandre Bergel 5/31/2010 22:55'!
stepTime
	"Update every 5 seconds to avoid hogging the system."
	^ 5000! !

StringMorph subclass: #MMMemoryUsageIndicadorMorph
	instanceVariableNames: 'prevUsage'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MemoryMonitor'!

!MMMemoryUsageIndicadorMorph methodsFor: 'as yet unclassified' stamp: 'Alexandre Bergel 5/31/2010 15:46'!
step
	| total used usedNow valueString |
	total := SmalltalkImage current vmParameterAt: 3.
	used := SmalltalkImage current vmParameterAt: 2.
	usedNow := used / total roundTo: 0.01.
	prevUsage = usedNow
		ifFalse: [super step.
			prevUsage := usedNow.
			"self done: prevUsage."
			valueString := ((used / (1024 * 1024)) asFloat roundTo: 0.1) asString , 'M/' , ((total / (1024 * 1024)) asFloat roundTo: 0.1) asString , 'M = ' , ((used / total roundTo: 0.001)
						* 100) asString , '%'.
			self contents: 'Memory usage : ', valueString]! !

!MMMemoryUsageIndicadorMorph methodsFor: 'as yet unclassified' stamp: 'Alexandre Bergel 5/31/2010 15:44'!
stepTime
	"Update every 5 seconds to avoid hogging the system."
	^ 5000! !

Object subclass: #MMProbe
	instanceVariableNames: 'values probe color'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MemoryMonitor'!

MMProbe subclass: #MEndOfMemoryProbe
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MemoryMonitor'!

!MEndOfMemoryProbe methodsFor: 'accessing' stamp: 'Alexandre Bergel 5/31/2010 15:35'!
color
	^ Color lightRed! !

!MEndOfMemoryProbe methodsFor: 'accessing' stamp: 'Alexandre Bergel 5/31/2010 15:35'!
currentEndOfMemory
	^ SmalltalkImage current vmParameterAt: 3! !

!MEndOfMemoryProbe methodsFor: 'initialization' stamp: 'Alexandre Bergel 5/31/2010 15:35'!
initialize
	super initializeFor: #currentEndOfMemory! !

MMProbe subclass: #MFreeMemoryProbe
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MemoryMonitor'!

!MFreeMemoryProbe methodsFor: 'accessing' stamp: 'Alexandre Bergel 5/25/2010 18:41'!
color
	^ Color lightRed! !

!MFreeMemoryProbe methodsFor: 'accessing' stamp: 'Alexandre Bergel 5/25/2010 18:37'!
currentFreeMemory
	| image memoryEnd youngSpaceEnd |
	image := SmalltalkImage current.
	memoryEnd			:= image vmParameterAt: 3.
	youngSpaceEnd		:= image vmParameterAt: 2.
	^ memoryEnd - youngSpaceEnd! !

!MFreeMemoryProbe methodsFor: 'initialization' stamp: 'Alexandre Bergel 5/25/2010 18:35'!
initialize
	super initializeFor: #currentFreeMemory! !

MMProbe subclass: #MMGCProbe
	instanceVariableNames: 'oldValue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MemoryMonitor'!

!MMGCProbe methodsFor: 'accessing' stamp: 'Alexandre Bergel 5/30/2010 21:36'!
color
	^ Color lightGray! !

!MMGCProbe methodsFor: 'accessing-readonly' stamp: 'Alexandre Bergel 5/27/2010 19:15'!
currentValue
	| v answer |

	v := SmalltalkImage current vmParameterAt: 9.
	"return true or false"
	
	oldValue ifNil: [ oldValue := v. ^ false ].
	answer := v - oldValue > 10.
	oldValue := v.
	^ answer! !

!MMGCProbe methodsFor: 'actions' stamp: 'Alexandre Bergel 5/27/2010 19:05'!
renderOnCanvas: aCanvas bounds: bounds
	| width origin height maxValue offset minValue diffExtremeValue diffExtremeValues step previousPoint newPoint oldPoint newPoint1 newPoint2 |
	origin := bounds origin.
	height := bounds height.
	width := bounds width.
	
	offset := 0 @ 0.
	step := (bounds width / self historyLength ).

	self values
		do: [ :v | 
			v ifTrue: [
				newPoint1 := origin + offset asIntegerPoint.
				newPoint2 := newPoint1 + (0 @ height).
				aCanvas line: newPoint1 to: newPoint2 color: self color ].
			offset := offset + (step @ 0) ]! !

!MMProbe class methodsFor: 'as yet unclassified' stamp: 'Alexandre Bergel 5/31/2010 15:20'!
probeBlock: aSelectorOrBlock
	^ self new initializeFor: aSelectorOrBlock! !

!MMProbe methodsFor: 'accessing' stamp: 'Alexandre Bergel 5/25/2010 18:27'!
addValue: v
	self shiftValues.
	self values at: self values size put: v! !

!MMProbe methodsFor: 'updating' stamp: 'Alexandre Bergel 5/25/2010 18:27'!
clear
	values := nil! !

!MMProbe methodsFor: 'accessing' stamp: 'Alexandre Bergel 5/25/2010 20:22'!
color
	^ color! !

!MMProbe methodsFor: 'accessing' stamp: 'Alexandre Bergel 5/25/2010 20:23'!
color: aColor
	color := aColor ! !

!MMProbe methodsFor: 'metrics' stamp: 'Alexandre Bergel 5/25/2010 18:25'!
computeMaxValue
	^ self values max! !

!MMProbe methodsFor: 'metrics' stamp: 'Alexandre Bergel 5/25/2010 18:25'!
computeMinValue
	^ self values min! !

!MMProbe methodsFor: 'accessing-readonly' stamp: 'Alexandre Bergel 5/25/2010 18:26'!
currentValue
	^ probe value: self! !

!MMProbe methodsFor: 'configuration' stamp: 'Alexandre Bergel 5/25/2010 18:22'!
historyLength
	^ 100! !

!MMProbe methodsFor: 'initialization' stamp: 'Alexandre Bergel 5/25/2010 20:22'!
initializeFor: aSelectorOrBlock
	| v |
	super initialize.
	color := Color green.
	probe := aSelectorOrBlock.
	values := Array new: self historyLength.
	v := self currentValue.
	1 to: values size do: [ :i | values at: i put: v ]! !

!MMProbe methodsFor: 'actions' stamp: 'Alexandre Bergel 5/25/2010 18:32'!
renderOnCanvas: aCanvas bounds: bounds
	| width origin height maxValue offset minValue diffExtremeValue diffExtremeValues step previousPoint newPoint oldPoint |
	origin := bounds origin.
	height := bounds height.
	width := bounds width.
	maxValue := self computeMaxValue.
	minValue := self computeMinValue.
	diffExtremeValues := maxValue - minValue.
	diffExtremeValues = 0 ifTrue: [ diffExtremeValues := 1 ].
	offset := 0 @ 0.
	step := (bounds width / self historyLength ).
	previousPoint := nil.
	self values
		do: [ :v | 
			maxValue = 0
				ifTrue: [ maxValue := 1 ].
				
			newPoint := origin + offset asIntegerPoint + (0 @ (height) - 1) - (0@((v - minValue) * height / diffExtremeValues) asInteger).
			oldPoint ifNotNil: [ aCanvas line: oldPoint to: newPoint color: self color ].
			oldPoint := newPoint.
			offset := offset + (step @ 0) ]! !

!MMProbe methodsFor: 'actions' stamp: 'Alexandre Bergel 5/25/2010 18:28'!
shiftValues
	2 to: self values size do: [ :i | self values at: i - 1 put: (self values at: i) ].
	self values at: (self values size) put: 0! !

!MMProbe methodsFor: 'actions' stamp: 'Alexandre Bergel 5/25/2010 18:29'!
updateValue
	self addValue: self currentValue! !

!MMProbe methodsFor: 'accessing' stamp: 'Alexandre Bergel 5/25/2010 18:29'!
values 
	| v |
	values ifNotNil: [ ^ values ].
	values := Array new: (self historyLength).
	v := self currentValue.
	1 to: self historyLength do: [ :i | values at: i put: v].
	
	^ values! !

MMProbe subclass: #MUsedMemoryProbe
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MemoryMonitor'!

!MUsedMemoryProbe methodsFor: 'accessing' stamp: 'Alexandre Bergel 5/25/2010 18:35'!
currentUsedMemory
	^ SmalltalkImage current vmParameterAt: 2! !

!MUsedMemoryProbe methodsFor: 'initialization' stamp: 'Alexandre Bergel 5/25/2010 18:35'!
initialize
	super initializeFor: #currentUsedMemory! !

Object subclass: #MemoryMonitor
	instanceVariableNames: 'graph memoryProbe'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MemoryMonitor'!

!MemoryMonitor commentStamp: '<historical>' prior: 0!
Manuia means "To your health!!" in Rapanui,  polynesian language used in Easter Island!

!MemoryMonitor class methodsFor: 'opening' stamp: 'Alexandre Bergel 5/31/2010 14:58'!
menuCommandOn: aBuilder
	<worldMenu>
	(aBuilder item: #'MemoryMonitor')
		parent: #Tools;
		action: [ self open ];
		icon: self taskbarIcon.! !

!MemoryMonitor class methodsFor: 'opening' stamp: 'Alexandre Bergel 5/26/2010 19:42'!
open
	^ self new open! !

!MemoryMonitor methodsFor: 'ui' stamp: 'Alexandre Bergel 5/31/2010 15:20'!
addProbe
	| className newProbe colors |
	className := UIManager default request: 'Enter the name of a class'.
	className ifNil: [ ^ self ].
	(Smalltalk globals includesKey: className asSymbol) ifFalse: [ self inform: 'class not found'. ^ self ].
	
	newProbe := MMProbe probeBlock: [ :probe | (Smalltalk at: className asSymbol) instanceCount].

	colors := Color wheel: 10.
	newProbe color: (colors at: graph numberOfProbes).
	
	graph addProbe: newProbe.! !

!MemoryMonitor methodsFor: 'ui' stamp: 'Alexandre Bergel 5/25/2010 20:03'!
buttonAddProbeLabel
	^ 'probe class'! !

!MemoryMonitor methodsFor: 'ui' stamp: 'Alexandre Bergel 5/25/2010 17:23'!
buttonClearLabel
	^ 'clear'! !

!MemoryMonitor methodsFor: 'ui' stamp: 'Alexandre Bergel 5/25/2010 17:39'!
buttonGarbageCollectLabel
	^ 'gc'! !

!MemoryMonitor methodsFor: 'ui' stamp: 'Alexandre Bergel 5/25/2010 17:40'!
buttonOn
	^ false! !

!MemoryMonitor methodsFor: 'ui' stamp: 'Alexandre Bergel 5/25/2010 17:24'!
clear
	graph clear! !

!MemoryMonitor methodsFor: 'ui' stamp: 'Alexandre Bergel 5/25/2010 17:39'!
garbageCollect
	Smalltalk garbageCollect! !

!MemoryMonitor methodsFor: 'initialize-release' stamp: 'Alexandre Bergel 5/31/2010 22:58'!
initialize
	super initialize.
	
	graph := MMGraph new.
	
	graph 
		addProbe: (memoryProbe := MUsedMemoryProbe new); 
		addProbe: MMGCProbe new;
		addProbe: MEndOfMemoryProbe new! !

!MemoryMonitor methodsFor: 'ui' stamp: 'Alexandre Bergel 5/25/2010 20:15'!
initializeButtonsIn: aWindow

	| clearButton garbageCollectButton addProbeButton |
	clearButton := PluggableButtonMorph
				on: self
				getState: #buttonOn
				action: #clear
				label: #buttonClearLabel.
	aWindow 
		addMorph: clearButton
		fullFrame: (LayoutFrame fractions: (0 @ 0 corner: 0.3 @ 0) offsets: (0 @ 0 corner: 0 @ 24)).

	garbageCollectButton := PluggableButtonMorph
				on: self
				getState: #buttonOn
				action: #garbageCollect
				label: #buttonGarbageCollectLabel.
	aWindow 
		addMorph: garbageCollectButton
		fullFrame: (LayoutFrame fractions: (0.3 @ 0 corner: 0.6 @ 0) offsets: (0 @ 0 corner: 0 @ 24)).

	addProbeButton := PluggableButtonMorph
				on: self
				getState: #buttonOn
				action: #addProbe
				label: #buttonAddProbeLabel.
	aWindow 
		addMorph: addProbeButton
		fullFrame: (LayoutFrame fractions: (0.6 @ 0 corner: 1 @ 0) offsets: (0 @ 0 corner: 0 @ 24)).
! !

!MemoryMonitor methodsFor: 'public' stamp: 'Alexandre Bergel 5/31/2010 22:58'!
open
	"
	self new open
	"
	| window stringMorph |
	window := StandardWindow  labelled: self windowTitle.

	window 
		addMorph: graph
		fullFrame: (LayoutFrame fractions: (0 @ 0 corner: 1 @ 0.6) offsets: (0 @ 24 corner: 0 @ 0)).

	window 
		addMorph: (stringMorph := MMMemoryUsageIndicadorMorph new)
		fullFrame: (LayoutFrame fractions: (0 @ 0.6 corner: 1 @ 0.8) offsets: (0 @ 0 corner: 0 @ 0)).

	stringMorph := MMMemoryBoundsIndicadorMorph new.
	stringMorph memoryProbe: memoryProbe.
	window 
		addMorph: (stringMorph)
		fullFrame: (LayoutFrame fractions: (0 @ 0.8 corner: 1 @ 1) offsets: (0 @ 0 corner: 0 @ 0)).

	window model: self.

	self initializeButtonsIn: window.
	window openInWorld.
	window extent: 400 @ 150.
	^ window! !

!MemoryMonitor methodsFor: 'configuration' stamp: 'Alexandre Bergel 5/25/2010 14:30'!
windowTitle
	^ 'Manuia Memory Monitor'! !
